library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity testdesign is

-- empty entity for a test bench

end testdesign;

architecture testing of testdesign is

-- All input and output signals will go here

-- We need:
--
--            - 100 Mhz clock (Nios2 processor)
--            - 18.432 MHz clock (Audio chip), this is AUD_CLK
--            - Data from the ADC output ???
--            - Read and Write signals from/to the Avalon bus ?
--            - address, byteenable and chipselect
--            - readdata, writedata to communicate with the Avalon Bus
--            - AUD_ADC_DAT : Raw 1-bit ADC data
--            - AUD_BCLK
--
--
--            Output signals to monitor activity


 signal   clk :  std_logic := '0';
 signal   reset_n :  std_logic;

    -- Bus master signals
 signal   address      :  std_logic_vector (7 downto 0);       --- What should this have for the case of the test bench?
 signal   byteenable   :  std_logic_vector (1 downto 0);    --- What should this have for the case of the test bench?
 signal   writedata    :  std_logic_vector (15 downto 0);    --- What should this have for the case of the test bench?
 signal   read         :  std_logic:='1';                        --- What should this have for the case of the test bench?
 signal   write        :  std_logic:='1';                        --- What should this have for the case of the test bench?   
 signal   chipselect   :  std_logic:='1';                        --- What should this have for the case of the test bench?

     -- Slave signals
 signal   readdata     :  std_logic_vector (15 downto 0);
 signal   waitrequest  :  std_logic:='0';    ---- This is not used ??
   
    -- Audio interface signals
 signal   AUD_CLK      :  std_logic:='0';                    --    18.43MHz audio clock AUD_XCK
 signal   AUD_ADCLRCK  :  std_logic:='0';                    --    Audio CODEC ADC LR Clock
 signal   AUD_ADCDAT   :  std_logic:='0';                    --    Audio CODEC ADC Data
 signal   AUD_DACLRCK  :  std_logic:='0';                    --    Audio CODEC DAC LR Clock
 signal   AUD_DACDAT   :  std_logic:='0';                    --    Audio CODEC DAC Data
 signal   AUD_BCLK     :  std_logic:='1';                     --    Audio CODEC Bit-Stream Clock -- Must make that
 signal   AUD_XCK      :  std_logic:='0';
    -- Test signals --
 signal   ledr           :  std_logic_vector (17 downto 0);
 signal   i            :  integer:=0;
 signal   t_aud_bclk   :std_logic :='0';
 
    -- ADC Data --  6 Different 16-bit samples, the input will have a sawtooth shape
 
signal adc_dat        :  std_logic_vector (95 downto 0) := "000000000000000000000000000000010000000000000010000000000000010000000000000010000000000000010000";


-- // Exercise 1 for tutorials // --

--        signal clk, resetn : std_logic;                             -- input signals
--        signal key : std_logic_vector (3 downto 0);                    -- key signals
--        signal hex6, hex5, hex4 : std_logic_vector (6 downto 0);
--        signal dummy : std_logic;      -- output signals
       
begin
 
--
U1: entity work.de2_audio port map(
    clk => clk,
    reset_n =>reset_n,
    address => address,
    byteenable => byteenable,
    writedata => writedata,
    read => read,
    write => write,
    chipselect => chipselect,
    readdata => readdata,
    waitrequest => waitrequest,
    AUD_CLK => AUD_CLK,
    AUD_ADCLRCK => AUD_ADCLRCK,
    AUD_ADCDAT => AUD_ADCDAT,
    AUD_DACLRCK => AUD_DACLRCK,
    AUD_DACDAT => AUD_DACDAT,
    AUD_BCLK => AUD_BCLK,
    --test_AUD_BCLK => t_aud_bclk,
    AUD_XCK => AUD_XCK,
    ledr => ledr
   
);

-- 100 MHz clock
process
begin
    clk <= '0';
    wait for 5 ns;

    loop
        clk <= '1';
        wait for 5 ns;
        clk <= '0';
        wait for 5 ns;
    end loop;
   
wait ;   
end process;

--clk <= not clk after 5 ns;   
   
-- 18.1 Mhz clock : AUDIO_CLOCK
   
--AUD_CLK <= not AUD_CLK after 27.8 ns;   
   
process
begin
    AUD_CLK <= '0';
    wait for 27.8 ns;
     
    loop
        AUD_CLK <= '1';
        wait for 27.8 ns;
        AUD_CLK <= '0';
        wait for 27.8 ns;
    end loop;
end process;   


--process
--begin
--    AUD_BCLK <= '0';
--    wait for 1.95315 us;
--     
--    loop
--        AUD_BCLK <= '1';
--        wait for 1.95315 us;
--        AUD_BCLK <= '0';
--        wait for 1.95315 ns;
--    end loop;
--end process;   
-- AUD_BCLK ??


-- Read and chipselect signals

--read <= not read after 10 ns;
--chipselect <= not chipselect after 10 ns;


process
begin
    chipselect <= '0';
    read <= '0';
    wait for 10 ns;   -- 2 clock periods
   
    loop
        chipselect <= '1';
        read <= '1';
        wait for 10 ns;
        chipselect <= '0';
        read <= '0';
        wait for 10 ns;
    end loop;
end process;
   
-- Reset signal always kept low


    address <= "00000001" ;   -- Arbitrary value ?
    --reset_n <= '1';
    byteenable <= "11";         -- Arbitrary value ?
--    chipselect <= '1';          -- Must be changed


-- Read and write


--    read  <= '1' ;   -- This is not correct, read MUST change
    write <= '0';


-- Need ADC data   
-- What is "writedata" supposed to be ?

    writedata <= "0000000000000000";   -- We do not care about the data written


-- AUD_BCLK 96kHz = 2(channels)* 8kHz (SR) * 16 (bits/sample) --> T/2 = 5.082 ns   

--AUD_BCLK <= not AUD_BCLK after 5.082 ns;
   
--process
--begin
--    AUD_BCLK <= '0' ;
--    wait for 1.953 us;
--    loop
--        AUD_BCLK <= '1';
--        wait for 1.953 ns;
--        AUD_BCLK <= '0';
--        wait for 1.953 ns;
--    end loop;
--
--end process;

-- Creating the ADC data

--i <= i + 1 after 10.41 ns;

--AUD_ADCDAT <= adc_dat(i);
   
process
begin
    i <= 0;
    wait for 5.082 ns;
    loop
    wait for 10.41 ns;  -- This is 1 /  the 16*AUD_CLK
    i <= i+1;
    AUD_ADCDAT <= adc_dat(i mod 96);   
    end loop;
end process;   
   
   
   
process
begin

reset_n <= '0';

wait for 10 ns;
reset_n <= '1';
wait;
end process;

-- The end--
end testing;        